@@ -6,12 +6,14 @@
 #ifndef __LIST_OF_LISTS__
 #define __LIST_OF_LISTS__
 
-typedef enum { INT_LIST, DBL_LIST, CHAR_LIST, LIST_LIST } list_element_type;
+typedef enum { INT_LIST, DBL_LIST, CHAR_LIST, MSA_PTR_LIST, GFF_PTR_LIST, 
+	       LIST_LIST } 
+  list_element_type;
 
 struct list_of_list_struct {
   List *lst;      //list of lists.  Each element can be be a List * 
-                  //containing ints, doubles, or char*.  OR an element can
-                  //be a ListOfList*.
+                  //containing ints, doubles, char*, or void*.  OR an element 
+                  //can be a ListOfList*.
   List *lstName;  //list of char* giving name of each element of lst
   List *lstType;  //list of list_element_type giving type of each element in lst
   char *class;    //NULL if this should be treated as a list; otherwise tells
@@ -54,8 +56,20 @@ void lol_push_treeModel(ListOfLists *lol, TreeModel *tm,
 void lol_push_gff(ListOfLists *lol, GFF_Set *gff,
 		  const char *name);
 
+void lol_push_gff_ptr(ListOfLists *lol, GFF_Set *gff, const char *name);
+
+void lol_push_msa_ptr(ListOfLists *lol, MSA *msa, const char *name);
+
 void lol_push_wig(ListOfLists *lol, double *scores, MSA *msa);
 
+void lol_push_dbl_array(ListOfLists *lol, void *data, char *name, 
+			int ndim, int *dimsize, char ***dimname);
+
+List *lol_find_list(ListOfLists *lol, const char *lstName, 
+		    list_element_type lstType);
+
+ListOfLists *lol_find_lol(ListOfLists *lol, const char *name);
+
 void lol_free(ListOfLists *lol);
 
 #endif
