@@ -29,49 +29,58 @@ Last updated: 1/5/2010
 
 #include <Rdefines.h>
 
+
 void rph_gff_free(SEXP gffPtr) {
-  gff_free_set((GFF_Set*)EXTPTR_PTR(gffPtr));
+  GFF_Set *gff = (GFF_Set*)EXTPTR_PTR(gffPtr);
+  rph_unregister_protected(gff);
+  gff_free_set(gff);
 }
 
 
 void rph_gff_feat_protect(GFF_Feature *feat) {
-  rph_protect_mem(feat);
-  rph_string_protect(feat->seqname);
-  rph_string_protect(feat->source);
-  rph_string_protect(feat->feature);
-  rph_string_protect(feat->attribute);
+  rph_mem_protect(feat);
+  rph_str_protect(feat->seqname);
+  rph_str_protect(feat->source);
+  rph_str_protect(feat->feature);
+  rph_str_protect(feat->attribute);
 }
 
+
 void rph_gff_protect(GFF_Set *gff) {
   int i;
-  rph_protect_mem(gff);
+  rph_mem_protect(gff);
   if (gff->features != NULL) {
     rph_lst_protect(gff->features);
     for (i=0; i < lst_size(gff->features); i++) {
       rph_gff_feat_protect(lst_get_ptr(gff->features, i));
     }
   }
-  rph_string_protect(gff->gff_version);
-  rph_string_protect(gff->source);
-  rph_string_protect(gff->source_version);
-  rph_string_protect(gff->date);
+  rph_str_protect(gff->gff_version);
+  rph_str_protect(gff->source);
+  rph_str_protect(gff->source_version);
+  rph_str_protect(gff->date);
   if (gff->groups != NULL) {
     rph_lst_protect(gff->groups);
     for (i=0; i < lst_size(gff->groups); i++) {
       GFF_FeatureGroup *g = lst_get_ptr(gff->groups, i);
-      rph_protect_mem(g);
-      rph_string_protect(g->name);
+      rph_mem_protect(g);
+      rph_str_protect(g->name);
       rph_lst_protect(g->features);
     }
   }
-  rph_string_protect(gff->group_tag);
+  rph_str_protect(gff->group_tag);
+}
+
+
+void rph_gff_register_protect(GFF_Set *gff) {
+  rph_register_protected_object(gff, (void (*)(void *))rph_gff_protect);
 }
 
 SEXP rph_gff_new_extptr(GFF_Set *gff) {
   SEXP result;
   PROTECT(result=R_MakeExternalPtr((void*)gff, R_NilValue, R_NilValue));
   R_RegisterCFinalizerEx(result, rph_gff_free, 1);
-  rph_gff_protect(gff);
+  rph_gff_register_protect(gff);
   UNPROTECT(1);
   return result;
 }
@@ -107,6 +116,7 @@ SEXP rph_gff_dataframe(SEXP gffPtr) {
 
 
   gff = (GFF_Set*)EXTPTR_PTR(gffPtr);
+  rph_gff_register_protect(gff);
 
   len = lst_size(gff->features);
 
@@ -229,7 +239,6 @@ SEXP rph_gff_dataframe(SEXP gffPtr) {
     }
   }
   SET_NAMES(result, header);
-  rph_gff_protect(gff);
 
   UNPROTECT(11);
   return result;
@@ -487,6 +496,7 @@ SEXP rph_gff_one_attribute(SEXP gffP, SEXP tagP) {
 
 
   if (lst_size(gff->features) == 0) return R_NilValue;
+  rph_gff_register_protect(gff);
   result = smalloc(maxResultLen*sizeof(char*));    
   tag = str_new_charstr(CHARACTER_VALUE(tagP));
   str_double_trim(tag);
@@ -527,7 +537,6 @@ SEXP rph_gff_one_attribute(SEXP gffP, SEXP tagP) {
     for (j=0; j < resultLen; j++) sfree(result[j]);
   }
   PROTECT(rv = rph_listOfLists_to_SEXP(lol));
-  rph_gff_protect(gff);
   UNPROTECT(1);
   return rv;
 }
@@ -543,6 +552,7 @@ SEXP rph_gff_overlapSelect(SEXP gffP, SEXP filter_gffP,
   double percentOverlap, overlappingFragments;
 
   gff = (GFF_Set*)EXTPTR_PTR(gffP);
+  rph_gff_register_protect(gff);
   filter_gff = (GFF_Set*)EXTPTR_PTR(filter_gffP);
   if (percentOverlapP == R_NilValue)
     percentOverlap = -1.0;
@@ -560,7 +570,6 @@ SEXP rph_gff_overlapSelect(SEXP gffP, SEXP filter_gffP,
   filter_gff = gff_overlap_gff(gff, filter_gff,
 			       numbaseOverlap, percentOverlap, nonOverlapping,
 			       overlappingFragments);
-  rph_gff_protect(gff);
   return rph_gff_new_extptr(filter_gff);
 }
 
@@ -568,9 +577,9 @@ SEXP rph_gff_overlapSelect(SEXP gffP, SEXP filter_gffP,
 SEXP rph_gff_add_UTRs(SEXP gffP) {
   GFF_Set *gff;
   gff = (GFF_Set*)EXTPTR_PTR(gffP);
+  rph_gff_register_protect(gff);
   gff_group(gff, "transcript_id");
   gff_create_utrs(gff);
-  rph_gff_protect(gff);
   return gffP;
 }
 
@@ -578,9 +587,9 @@ SEXP rph_gff_add_UTRs(SEXP gffP) {
 SEXP rph_gff_add_introns(SEXP gffP) {
   GFF_Set *gff;
   gff = (GFF_Set*)EXTPTR_PTR(gffP);
+  rph_gff_register_protect(gff);
   gff_group(gff, "transcript_id");
   gff_create_introns(gff);
-  rph_gff_protect(gff);
   return gffP;
 }
 
@@ -588,9 +597,9 @@ SEXP rph_gff_add_introns(SEXP gffP) {
 SEXP rph_gff_add_signals(SEXP gffP) {
   GFF_Set *gff;
   gff = (GFF_Set*)EXTPTR_PTR(gffP);
+  rph_gff_register_protect(gff);
   gff_group(gff, "transcript_id");
   gff_create_signals(gff);
-  rph_gff_protect(gff);
   return gffP;
 }
 
@@ -598,9 +607,9 @@ SEXP rph_gff_add_signals(SEXP gffP) {
 SEXP rph_gff_fix_start_stop(SEXP gffP) {
   GFF_Set *gff;
   gff = (GFF_Set*)EXTPTR_PTR(gffP);
+  rph_gff_register_protect(gff);
   gff_group(gff, "transcript_id");
   gff_fix_start_stop(gff);
-  rph_gff_protect(gff);
   return gffP;
 }
 
@@ -610,10 +619,9 @@ SEXP rph_gff_fix_start_stop(SEXP gffP) {
 SEXP rph_gff_inverse(SEXP gffP, SEXP regionP) {
   GFF_Set *gff, *region, *notgff;
   gff = (GFF_Set*)EXTPTR_PTR(gffP);
-  region = (GFF_Set*)EXTPTR_PTR(regionP);
-  
+  rph_gff_register_protect(gff);
+  region = (GFF_Set*)EXTPTR_PTR(regionP);  
   notgff = gff_inverse(gff, region);
-  rph_gff_protect(gff);
   return rph_gff_new_extptr(notgff);
 }
 
@@ -628,9 +636,10 @@ SEXP rph_gff_featureBits(SEXP gffListP, SEXP orP, SEXP returnGffP) {
   numGff = length(gffListP);
   gfflist = lst_new_ptr(numGff);
   //  Rf_PrintValue(gffListP);
-  for (i = 0; i < length(gffListP); i++) {
+  for (i = 0; i < numGff; i++) {
     gff = (GFF_Set*)EXTPTR_PTR(VECTOR_ELT(gffListP, i));
     lst_push_ptr(gfflist, gff);
+    rph_gff_register_protect(gff);
   }
   or = LOGICAL_VALUE(orP);
   returnGff = LOGICAL_VALUE(returnGffP);
@@ -661,8 +670,6 @@ SEXP rph_gff_featureBits(SEXP gffListP, SEXP orP, SEXP returnGffP) {
     }
     numbit = gff_flatten_mergeAll(newgff);
   }
-  for (i=0; i < numGff; i++)
-    rph_gff_protect((GFF_Set*)lst_get_ptr(gfflist, i));
   if (returnGff) 
     return rph_gff_new_extptr(newgff);
   PROTECT(rv = allocVector(INTSXP, 1));
@@ -677,12 +684,12 @@ SEXP rph_gff_append(SEXP gffListP) {
   int i, j;
   for (i=0 ; i<length(gffListP); i++) {
     gff = (GFF_Set*)EXTPTR_PTR(VECTOR_ELT(gffListP, i));
+    rph_gff_register_protect(gff);
     for (j=0; j < lst_size(gff->features); j++) {
       checkInterruptN(j, 1000);
       lst_push_ptr(newgff->features, 
 		   gff_new_feature_copy(lst_get_ptr(gff->features, j)));
     }
-    rph_gff_protect(gff);
   }
   return rph_gff_new_extptr(newgff);
 }
@@ -692,6 +699,7 @@ SEXP rph_gff_split(SEXP gffP, SEXP maxLengthP, SEXP dropP, SEXP splitFromRightP)
   GFF_Set *gff, *newgff;
   int *maxlen, maxlen_size, *splitFromRight, splitFrom_size, drop;
   gff = (GFF_Set*)EXTPTR_PTR(gffP);
+  rph_gff_register_protect(gff);
   drop = LOGICAL_VALUE(dropP);
   PROTECT(maxLengthP = AS_INTEGER(maxLengthP));
   maxlen = INTEGER_POINTER(maxLengthP);
@@ -701,7 +709,6 @@ SEXP rph_gff_split(SEXP gffP, SEXP maxLengthP, SEXP dropP, SEXP splitFromRightP)
   splitFrom_size = LENGTH(splitFromRightP);
   newgff = gff_split(gff, maxlen, maxlen_size, drop, 
 		     splitFromRight, splitFrom_size);
-  rph_gff_protect(gff);
   UNPROTECT(2);
   return rph_gff_new_extptr(newgff);
 }
@@ -709,8 +716,8 @@ SEXP rph_gff_split(SEXP gffP, SEXP maxLengthP, SEXP dropP, SEXP splitFromRightP)
 
 SEXP rph_gff_sort(SEXP gffP) {
   GFF_Set *gff = (GFF_Set*)EXTPTR_PTR(gffP);
+  rph_gff_register_protect(gff);
   gff_sort(gff);
-  rph_gff_protect(gff);
   return gffP;
 }
 
@@ -718,8 +725,8 @@ SEXP rph_gff_sort(SEXP gffP) {
 SEXP rph_gff_nonOverlapping_genes(SEXP gffP) {
   GFF_Set *gff = (GFF_Set*)EXTPTR_PTR(gffP);
   if (lst_size(gff->features) == 0) return gffP;
+  rph_gff_register_protect(gff);
   gff_group(gff, "transcript_id");
   gff_remove_overlaps(gff, NULL);
-  rph_gff_protect(gff);
   return gffP;
 }
