@@ -72,6 +72,7 @@ SEXP rph_phyloFit(SEXP msaP,
   if (initModP != R_NilValue) {
     pf->input_mod = (TreeModel*)EXTPTR_PTR(initModP);
     pf->subst_mod = pf->input_mod->subst_mod;
+    rph_tm_register_protect(pf->input_mod);
   } else pf->subst_mod = rph_get_subst_mod(substModP);
   
   pf->estimate_scale_only = LOGICAL_VALUE(scaleOnlyP);
@@ -95,9 +96,6 @@ SEXP rph_phyloFit(SEXP msaP,
       lst_push_dbl(pf->rate_consts, doubleP[i]);
   }
 
-  if (initModP != R_NilValue) 
-    pf->input_mod = (TreeModel*)EXTPTR_PTR(initModP);
-
   pf->random_init = LOGICAL_VALUE(initRandomP);
 
   pf->init_backgd_from_data = LOGICAL_VALUE(initBackgdFromDataP);
@@ -121,8 +119,10 @@ SEXP rph_phyloFit(SEXP msaP,
     goto rph_phyloFit_end;
   }
 
-  if (gffP != R_NilValue)
+  if (gffP != R_NilValue) {
     pf->gff = (GFF_Set*)EXTPTR_PTR(gffP);
+    rph_gff_register_protect(pf->gff);
+  }
 
   if (ninfSitesP != R_NilValue)
     pf->nsites_threshold = INTEGER_VALUE(ninfSitesP);
@@ -167,10 +167,9 @@ SEXP rph_phyloFit(SEXP msaP,
     pf->selection = NUMERIC_VALUE(selectionP);
   }
   
+  rph_msa_register_protect(pf->msa);
+
   run_phyloFit(pf);
-  rph_msa_protect(pf->msa);
-  if (pf->gff != NULL)
-    rph_gff_protect(pf->gff);
   rv = PROTECT(rph_listOfLists_to_SEXP(pf->results));
   numProtect++;
 
