@@ -381,10 +381,13 @@ mod3 <- mod
 mod3$backgd <- c(0.6, 0.1, 0.2, 0.1)
 m <- simulate.msa(mod, 20)
 m <- simulate.msa(list(mod, mod2, mod3), 20, hmm=h)
-m <- matrix(1, nrow=9, ncol=9)
+m <- matrix(1, nrow=3, ncol=3)
 h <- hmm(m)
-m <- simulate.msa(list(mod, mod2, mod3, mod, mod2, mod3, mod, mod2, mod3),
-                  20, hmm=h)
+l <- simulate.msa(list(mod, mod2, mod3), 100, get.features=TRUE, hmm=h)
+names(l)
+l$msa
+l$feats
+coverage.feat(l$feats[l$feats$feature=="state1",])
 unlink(filename)
 
 #' get4d.msa
@@ -415,7 +418,7 @@ require("rphast")
 exampleArchive <- system.file("extdata", "examples.zip", package="rphast")
 unzip(exampleArchive, "ENr334.maf")
 m <- read.msa("ENr334.maf")
-mod <- phyloFit(m, tree="((human,(mm9,rn4)),canFam2)")
+mod <- phyloFit(m, tree="((hg18,(mm9,rn4)),canFam2)")
 x <- postprob.msa(sub.msa(m, start.col=41447839, end.col=41448033, refseq="hg18"), mod)
 dim(x)
 dimnames(x)
@@ -426,7 +429,7 @@ require("rphast")
 exampleArchive <- system.file("extdata", "examples.zip", package="rphast")
 unzip(exampleArchive, "ENr334.maf")
 m <- read.msa("ENr334.maf")
-mod <- phyloFit(m, tree="((human,(mm9,rn4)),canFam2)")
+mod <- phyloFit(m, tree="((hg18,(mm9,rn4)),canFam2)")
 x <- expected.subs.msa(sub.msa(m, start.col=41447839, end.col=41448033, refseq="hg18"), mod)
 dim(x)
 dimnames(x)
@@ -438,12 +441,23 @@ require("rphast")
 exampleArchive <- system.file("extdata", "examples.zip", package="rphast")
 unzip(exampleArchive, "ENr334.maf")
 m <- read.msa("ENr334.maf")
-mod <- phyloFit(m, tree="((human,(mm9,rn4)),canFam2)")
+mod <- phyloFit(m, tree="((hg18,(mm9,rn4)),canFam2)")
 x <- total.expected.subs.msa(sub.msa(m, start.col=41447839, end.col=41448033, refseq="hg18"), mod)
 dim(x)
 dimnames(x)
 x["mm9-rn4",,]
 
+#' col.expected.subs.msa
+require("rphast")
+exampleArchive <- system.file("extdata", "examples.zip", package="rphast")
+unzip(exampleArchive, "ENr334.maf")
+m <- read.msa("ENr334.maf")
+mod <- phyloFit(m, tree="((hg18,(mm9,rn4)),canFam2)")
+x <- col.expected.subs.msa(sub.msa(m, start.col=41447839, end.col=41448033, refseq="hg18"), mod)
+dim(x)
+dimnames(x)
+x["mm9-rn4","CCTT",,]
+
 
 #' plot.msa
 require("rphast")
@@ -459,5 +473,15 @@ plot.msa(m[,1:50], refseq=NULL)
 rm(m)
 unlink("ENr334.maf")
 
+
+#' split.by.feature.msa
+require("rphast")
+exampleArchive <- system.file("extdata", "examples.zip", package="rphast")
+unzip(exampleArchive, c("ENr334.maf", "gencode.ENr334.gff"))      
+m <- read.msa("ENr334.maf")
+feats <- read.feat("gencode.ENr334.gff")
+feats$seqname <- "hg18"
+cdsAlign <- split.by.feature.msa(m, feats[feats$feature=="CDS",])
+
 rm(list = ls())
 gc()
