@@ -102,6 +102,19 @@ plot.rate.matrix(tm[["rate.matrix"]], eq.freq=tm[["backgd"]], filled=FALSE, alph
 unlink(filename)
 
 
+#' mod.backgd.tm
+require("rphast")
+exampleArchive <- system.file("extdata", "examples.zip", package="rphast")
+filename <- "rev.mod"
+unzip(exampleArchive, filename)
+tm <- read.tm(filename)
+#'
+# change background frequencies to new value, adjusting rate matrix
+mod.backgd.tm(tm, c(0.25, 0.25, 0.25, 0.25))
+#'
+# change background frequencies so that GC content is 0.6
+mod.backgd.tm(tm, gc=0.6)
+
 rm(list=ls())
 gc()
 
